#!/usr/bin/perl -w
#Program name = miniBlog
#Filename = miniBlog.pl
#Version 0.1
#RJJ Sat May  4 23:57:31 2002
#Richard Jacob
#r-jacob@pacbell.net
#7405 Fairmount Ave
#El Cerrito
#CA 94530 USA
#Copyright (C)
#
#miniBlog.pl is freely available as described in the Artistic License 
#which accompanies the program.

#Description:
#A mini Blog program that reads in a directory of text files and generates blog pages and archives by useing a template.
#Run from cron every 1 hour or so. Based on ideas from blosxom.pl.

#Input:
#input directory
#number of entries per a page
#output directory

#Output:
#Main page
#archive pages

use strict;
use File::stat;
use Time::localtime;
use HTML::Template;

# --- Configurable variables -----

# What's my blog's title?
my $blog_title = 'my Blog';

# Where are my blog entries kept?
my $datadir = "/home/www/myBlog/blog/";

#Where is the output template kept?
my $templateLocation = "/home/www/myBlog/templates/main.dwt";

# How many entries should I show on the home page?
my $num_entries = 10;

# Where is the main page kept?
my $rootdir = "/home/www/myBlog/";

# Where are the archive pages kept?
my $archivedir = "/home/www/myBlog/archives/";

#name of the default page
my $defaultpage = "default"

#name of the archive pages
my $archivepage = "archive_"

#name of the archive pages directory
my $relativearchivedir = "archives/"

# --- End configurable variables -----

#Find entries
opendir (BLOGDIR, "$datadir");
my @blog_entries = readdir (BLOGDIR);
closedir (BLOGDIR);
#print "@blog_entries\n";

# Read in Entries
my $curdate = '';
my %blogEntries = ();
my $entry_index = 0;

foreach my $filename (@blog_entries){
	if ($filename =~ /\.txt$/){
		$entry_index ++;
		my($fn) = ($filename =~ /^(.*)\.txt$/);
		# Date fiddling for by-{year,month,day} archive views
		my $mtime = ctime(stat("$datadir/$filename")->mtime);
		my($dw,$mo,$da,$ti,$yr) = ( $mtime =~ /(\w{3}) +(\w{3}) +(\d{1,2}) +(\d{2}:\d{2}):\d{2} +(\d{4})$/ );
		#print "$filename $dw,$mo,$da,$ti,$yr\n";#bugcheck
		open (ENTRY, "$datadir/$filename")|| die "cant open peak list\n";
		my @blogEntry = <ENTRY>;
		close ENTRY;
		#place entries into a hash
		@blogEntry = ($dw,$mo,$da,$ti,$yr,$fn, @blogEntry);
		$blogEntries{$entry_index} = [@blogEntry];
	}
}

#Sort entries by date
my @sorted_entry_index = sort by_date keys(%blogEntries);
@sorted_entry_index = reverse @sorted_entry_index;

#define variables
my ($pagetitle, $Intro, $pagetext, $lastupdate, $pagenumber, $htmlfilename, $pageopen, %page_info, @page_info_array);
$pagenumber = 0;

#Set the lastentry data
my ($min, $hours, $day, $month, $year)= @{(localtime)}[1,2,3,4,5];
#print "$min, $hours, $day, $month, $year\n";
$year = $year +1900;
my (%month_name_hash) = &months;
my %month_name_hash_rev = reverse %month_name_hash;
my $month_name = $month_name_hash_rev{$month+1};
$lastupdate = "$month_name $day, $year $hours:$min";

#print out entries x per a page with smartlinks between archive
#open page
#($pagetitle, $Intro, $pagetext) = &open_page($pagenumber);
$pageopen = 0;
my (@current_entry, $new_pagetext, $entry_counter);
$entry_counter = 1;
foreach my $sortedEntry (@sorted_entry_index){
	#print title,text, date
	if ($pageopen == 0){
		($pagetitle, $Intro, $pagetext) = &open_page($pagenumber);
		$pageopen = 1;
	}
	@current_entry =();
	@current_entry = @{$blogEntries{$sortedEntry}};
	($new_pagetext) = &assemble_entry(@current_entry);
	$pagetext = $pagetext . $new_pagetext;
	if ($entry_counter == $num_entries){
		if ($pagenumber == 0){
			$htmlfilename = $rootdir . $defaultpage .".htm";
		}
		else{
			$htmlfilename = $archivedir . $archivepage . $pagenumber . ".htm";
		}
		$entry_counter = 0;
		$pagetext = $pagetext. "</table>";
		#$lastupdate = localtime;
		@page_info_array = ($pagetitle, $Intro, $pagetext, $lastupdate, $templateLocation, $htmlfilename);
		$page_info{$pagenumber} = [@page_info_array];
		$pagenumber ++;
		$pageopen = 0;
		@page_info_array = ();
	}
	$entry_counter ++;
}
if ($pageopen ==1){
	$pagetext = $pagetext. "</table>";
	if ($pagenumber == 0){
		$htmlfilename = $rootdir. $defaultpage .".htm";
	}
	else{
		$htmlfilename = $archivedir. $archivepage . $pagenumber . ".htm";
	}
	#$lastupdate = localtime;
	@page_info_array = ($pagetitle, $Intro, $pagetext, $lastupdate, $templateLocation, $htmlfilename);
	$page_info{$pagenumber} = [@page_info_array];
}
#Add links to pages
my ($links, $links_archive);
if ($pagenumber <= 1){
	$links ="";
	$links_archive ="";
}
else{
	$links = "<p align=\"center\"><a href=\"". $defaultpage .".htm\">Home</a>, Archive pages ";
	$links_archive = "<p align=\"center\"><a href=\"../". $defaultpage .".htm\">Home</a>, Archive pages ";
	for (my $k=1; $k < $pagenumber; $k++){
		$links = $links . "<a href=\"" .$relativearchivedir . $archivepage . $k . ".htm\">$k</a> ";
		$links_archive = $links_archive . "<a href=\"". $archivepage  . $k . ".htm\">$k</a> "
	}
}
foreach my $curr_page (keys %page_info){
	if ($curr_page == 0){
		$page_info{$curr_page}[2] = $page_info{$curr_page}[2] . $links;
	}
	else {
		$page_info{$curr_page}[2] = $page_info{$curr_page}[2] . $links_archive;
	}
	&close_page (@{$page_info{$curr_page}});		
}


#####################S U B  R O U T I N E S#######################################

sub by_date {
	my (%months_hash) = &months;
	($blogEntries{$a}[4] <=> $blogEntries{$b}[4]) ||($months_hash{$blogEntries{$a}[1]} <=> $months_hash{$blogEntries{$b}[1]})||($blogEntries{$a}[2] <=> $blogEntries{$b}[2])||($blogEntries{$a}[3] cmp $blogEntries{$b}[3]);
}

sub months {
	my %months_hash = ("Jan","1","Feb","2","Mar","3","Apr","4","May","5","Jun","6","Jul","7","Aug","8","Sep","9","Oct","10","Nov","11","Dec","12",);
	return %months_hash;
}

sub assemble_entry {
	my (@current_entry_sub) = @_;
	my $no_of_lines = @current_entry_sub;
	my $pagetext_sub = "<tr> <td>";
	$pagetext_sub = $pagetext_sub. "<h2>$current_entry[6]</h2>";
	$pagetext_sub = $pagetext_sub. "</td></tr><tr> <td>";
	for (my $j= 7; $j < $no_of_lines; $j++){
		$pagetext_sub = $pagetext_sub. "$current_entry[$j]";
	}
	$pagetext_sub = $pagetext_sub. "</td></tr><tr>";
	$pagetext_sub = $pagetext_sub. "<td>Posted : $current_entry[3] $current_entry[0], $current_entry[1] $current_entry[2], $current_entry[4]<p>&nbsp;</p></td>";
    $pagetext_sub = $pagetext_sub. "</tr>";
	return ($pagetext_sub);
}

sub open_page {
	my ($pagenumber_gen) = @_;
	my ($pagetitle_gen, $Intro_gen, $pagetext_gen);
	if ($pagenumber_gen == 0){
		$pagetitle_gen = "LostCulture";
		$Intro_gen = "<h2>A quick introduction</h2><p>Your text here</p>";
	}
	else {
		$pagetitle_gen = "Archive page $pagenumber_gen";
		$Intro_gen = "<h2>Archive page $pagenumber_gen</h2><p>Here are the archives from the last few months</p>";
	}
	#open table at start of page
	$pagetext_gen = "<table width=\"90%\" border=\"0\">";
	return ($pagetitle_gen, $Intro_gen, $pagetext_gen);
}

sub close_page {
	my ($pagetitle_cls, $Intro_cls, $pagetext_cls, $lastupdate_cls, $templateLocation_cls, $htmlfilename_cls) = @_;
	my $template = HTML::Template->new(filename => $templateLocation);
	#assembl the template
	$template->param(pagetitle => $pagetitle_cls);
	$template->param(intro => $Intro_cls);
	$template->param(main => $pagetext_cls);
	$template->param(lastupdate => $lastupdate_cls); 
	# print the template
	#print "$htmlfilename_cls\n";#bugcheck
	open (OUTPUT, ">$htmlfilename_cls");
	print OUTPUT $template->output;
	close OUTPUT;
	return ();
}
